@@ -23,6 +23,8 @@
 
 #include "lib/getopt.h"
 
+#include "fio_version.h"
+
 #if FIO_PATCH > 0
 static char fio_version_string[] =	__fio_stringify(FIO_MAJOR) "."	\
 					__fio_stringify(FIO_MINOR) "."	\
@@ -69,6 +71,7 @@ unsigned int fio_debug_jobno = -1;
 unsigned int *fio_debug_jobp = NULL;
 
 static char cmd_optstr[256];
+static int did_arg;
 
 const fio_fp64_t def_percentile_list[FIO_IO_U_LIST_MAX_LEN] = {
 	{ .u.f	=  1.0 },
@@ -187,18 +190,13 @@ static struct option l_opts[FIO_NR_OPTIONS] = {
 	},
 	{
 		.name		= (char *) "server",
-		.has_arg	= no_argument,
+		.has_arg	= optional_argument,
 		.val		= 'S',
 	},
 	{	.name		= (char *) "daemonize",
 		.has_arg	= no_argument,
 		.val		= 'D',
 	},
-	{
-		.name		= (char *) "net-port",
-		.has_arg	= required_argument,
-		.val		= 'P',
-	},
 	{
 		.name		= (char *) "client",
 		.has_arg	= required_argument,
@@ -1099,7 +1097,7 @@ static int fill_def_thread(void)
 
 static void usage(const char *name)
 {
-	printf("%s\n", fio_version_string);
+	printf("fio %s\n", fio_version_string);
 	printf("%s [options] [job options] <job file(s)>\n", name);
 	printf("\t--debug=options\tEnable debug logging\n");
 	printf("\t--output\tWrite output to file\n");
@@ -1122,9 +1120,8 @@ static void usage(const char *name)
 		" (def 1024)\n");
 	printf("\t--warnings-fatal Fio parser warnings are fatal\n");
 	printf("\t--max-jobs\tMaximum number of threads/processes to support\n");
-	printf("\t--server\tStart a backend fio server\n");
+	printf("\t--server=args\tStart a backend fio server\n");
 	printf("\t--client=hostname Talk to remove backend fio server at hostname\n");
-	printf("\t--net-port=port\tUse specified port for client/server connection\n");
 	printf("\nFio was written by Jens Axboe <jens.axboe@oracle.com>");
 	printf("\n                   Jens Axboe <jaxboe@fusionio.com>\n");
 }
@@ -1246,7 +1243,7 @@ static int client_flag_set(char c)
 	return 0;
 }
 
-int parse_cmd_client(char *client, char *opt)
+int parse_cmd_client(void *client, char *opt)
 {
 	return fio_client_add_cmd_option(client, opt);
 }
@@ -1257,7 +1254,7 @@ int parse_cmd_line(int argc, char *argv[])
 	int c, ini_idx = 0, lidx, ret = 0, do_exit = 0, exit_val = 0;
 	char *ostr = cmd_optstr;
 	int daemonize_server = 0;
-	char *cur_client = NULL;
+	void *cur_client = NULL;
 	int backend = 0;
 
 	/*
@@ -1267,6 +1264,8 @@ int parse_cmd_line(int argc, char *argv[])
 	optind = 1;
 
 	while ((c = getopt_long_only(argc, argv, ostr, l_opts, &lidx)) != -1) {
+		did_arg = 1;
+
 		if ((c & FIO_CLIENT_FLAG) || client_flag_set(c)) {
 			if (parse_cmd_client(cur_client, argv[optind - 1])) {
 				exit_val = 1;
@@ -1301,14 +1300,16 @@ int parse_cmd_line(int argc, char *argv[])
 			terse_output = 1;
 			break;
 		case 'h':
-			if (!cur_client)
+			if (!cur_client) {
 				usage(argv[0]);
-			do_exit++;
+				do_exit++;
+			}
 			break;
 		case 'c':
-			if (!cur_client)
+			if (!cur_client) {
 				fio_show_option_help(optarg);
-			do_exit++;
+				do_exit++;
+			}
 			break;
 		case 's':
 			dump_cmdline = 1;
@@ -1317,9 +1318,10 @@ int parse_cmd_line(int argc, char *argv[])
 			read_only = 1;
 			break;
 		case 'v':
-			if (!cur_client)
-				log_info("%s\n", fio_version_string);
-			do_exit++;
+			if (!cur_client) {
+				log_info("fio %s\n", fio_version_string);
+				do_exit++;
+			}
 			break;
 		case 'V':
 			terse_version = atoi(optarg);
@@ -1404,15 +1406,14 @@ int parse_cmd_line(int argc, char *argv[])
 				exit_val = 1;
 				break;
 			}
+			if (optarg)
+				fio_server_set_arg(optarg);
 			is_backend = 1;
 			backend = 1;
 			break;
 		case 'D':
 			daemonize_server = 1;
 			break;
-		case 'P':
-			fio_net_port = atoi(optarg);
-			break;
 		case 'C':
 			if (is_backend) {
 				log_err("fio: can't be both client and server\n");
@@ -1420,8 +1421,12 @@ int parse_cmd_line(int argc, char *argv[])
 				exit_val = 1;
 				break;
 			}
-			fio_client_add(optarg);
-			cur_client = optarg;
+			if (fio_client_add(optarg, &cur_client)) {
+				log_err("fio: failed adding client %s\n", optarg);
+				do_exit++;
+				exit_val = 1;
+				break;
+			}
 			break;
 		default:
 			do_exit++;
@@ -1451,7 +1456,7 @@ int parse_cmd_line(int argc, char *argv[])
 			ret = add_job(td, td->o.name ?: "fio", 0);
 	}
 
-	while (optind < argc) {
+	while (!ret && optind < argc) {
 		ini_idx++;
 		ini_file = realloc(ini_file, ini_idx * sizeof(char *));
 		ini_file[ini_idx - 1] = strdup(argv[optind]);
@@ -1504,10 +1509,17 @@ int parse_options(int argc, char *argv[])
 			return 0;
 		if (is_backend || nr_clients)
 			return 0;
+		if (did_arg)
+			return 0;
 
 		log_err("No jobs(s) defined\n\n");
-		usage(argv[0]);
-		return 1;
+
+		if (!did_arg) {
+			usage(argv[0]);
+			return 1;
+		}
+
+		return 0;
 	}
 
 	if (def_thread.o.gtod_offload) {
@@ -1516,6 +1528,6 @@ int parse_options(int argc, char *argv[])
 		fio_gtod_cpu = def_thread.o.gtod_cpu;
 	}
 
-	log_info("%s\n", fio_version_string);
+	log_info("fio %s\n", fio_version_string);
 	return 0;
 }
