@@ -309,6 +309,9 @@ static int handle_probe_cmd(struct fio_net_cmd *cmd)
 	probe.fio_minor = FIO_MINOR;
 	probe.fio_patch = FIO_PATCH;
 
+	probe.os	= FIO_OS;
+	probe.arch	= FIO_ARCH;
+
 	return fio_net_send_cmd(server_fd, FIO_NET_CMD_PROBE, &probe, sizeof(probe));
 }
 
@@ -745,7 +748,7 @@ int fio_server_parse_string(const char *str, char **ptr, int *is_sock,
 {
 	*ptr = NULL;
 	*is_sock = 0;
-	*port = 0;
+	*port = fio_net_port;
 
 	if (!strncmp(str, "sock:", 5)) {
 		*ptr = strdup(str + 5);
@@ -799,7 +802,6 @@ int fio_server_parse_string(const char *str, char **ptr, int *is_sock,
 
 			hent = gethostbyname(host);
 			if (!hent) {
-				printf("FAIL\n");
 				free(*ptr);
 				*ptr = NULL;
 				return 1;
@@ -829,22 +831,25 @@ int fio_server_parse_string(const char *str, char **ptr, int *is_sock,
  */
 static int fio_handle_server_arg(void)
 {
-	int is_sock, ret;
+	int port = fio_net_port;
+	int is_sock, ret = 0;
 
 	saddr_in.sin_addr.s_addr = htonl(INADDR_ANY);
-	saddr_in.sin_port = htons(fio_net_port);
 
 	if (!fio_server_arg)
-		return 0;
+		goto out;
 
 	ret = fio_server_parse_string(fio_server_arg, &bind_sock, &is_sock,
-					&fio_net_port, &saddr_in.sin_addr);
+					&port, &saddr_in.sin_addr);
 
 	if (!is_sock && bind_sock) {
 		free(bind_sock);
 		bind_sock = NULL;
 	}
 
+out:
+	fio_net_port = port;
+	saddr_in.sin_port = htons(port);
 	return ret;
 }
 
